"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const mfs = require("m-fs");
const nodepath = require("path");
const defs_1 = require("../../defs");
const common_1 = require("./common");
const escapeHTML = require('escape-html');
const node_rename_path_1 = require("node-rename-path");
class TitleExtractor {
    static titleFromFileAsync(absFile) {
        return __awaiter(this, void 0, void 0, function* () {
            const content = yield mfs.readTextFileAsync(absFile);
            return yield this.titleFromFileContentAsync(content, absFile);
        });
    }
    static titleFromFileContentAsync(content, logFile) {
        return __awaiter(this, void 0, void 0, function* () {
            const firstLine = content.split('\n').shift();
            if (!firstLine) {
                throw new Error(`Cannot find a valid title in file: "${logFile}"`);
            }
            let title = firstLine;
            title = title.substr(1);
            return (title || '').trim();
        });
    }
    static titleFromDirAsync(absDir) {
        return __awaiter(this, void 0, void 0, function* () {
            const file = nodepath.join(absDir, defs_1.default.src.titleFile);
            let displayTitle = '';
            if (yield mfs.fileExists(file)) {
                displayTitle = yield mfs.readTextFileAsync(file);
            }
            else {
                displayTitle = nodepath.basename(absDir);
            }
            return (displayTitle || '').trim();
        });
    }
}
class TitleManager {
    constructor(config, state, pathManager) {
        this.config = config;
        this.state = state;
        this.pathManager = pathManager;
    }
    updateFileTitleAsync(relFile) {
        return __awaiter(this, void 0, void 0, function* () {
            const state = this.state;
            if (state.fileTitle[relFile]) {
                return new common_1.Result(state.fileTitle[relFile], true);
            }
            const path = this.pathManager.srcPath(relFile);
            const title = yield TitleExtractor.titleFromFileAsync(path);
            state.fileTitle[relFile] = title;
            yield this.writeFileTitleFile(relFile, title);
            return new common_1.Result(title, false);
        });
    }
    updateDirTitleAsync(relDir, recursive) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!relDir) {
                return new common_1.Result('', false);
            }
            const state = this.state;
            if (state.dirTitle[relDir]) {
                return new common_1.Result(state.dirTitle[relDir], true);
            }
            const path = this.pathManager.srcPath(relDir);
            const title = yield TitleExtractor.titleFromDirAsync(path);
            state.dirTitle[relDir] = title;
            yield this.writeDirTitleFile(relDir, title);
            if (recursive) {
                const parentDir = this.pathManager.basePath(relDir);
                yield this.updateDirTitleAsync(parentDir, true);
            }
            return new common_1.Result(title, false);
        });
    }
    writeFileTitleFile(relFile, title) {
        return __awaiter(this, void 0, void 0, function* () {
            const unescapedFile = node_rename_path_1.default(this.pathManager.destPath(relFile), (_) => {
                return { ext: defs_1.default.dest.titleExt };
            });
            yield mfs.writeFileAsync(unescapedFile, title);
            const escapedFile = node_rename_path_1.default(this.pathManager.destPath(relFile), (_) => {
                return { ext: defs_1.default.dest.titleHtmlExt };
            });
            yield mfs.writeFileAsync(escapedFile, escapeHTML(title));
        });
    }
    writeDirTitleFile(relDir, title) {
        return __awaiter(this, void 0, void 0, function* () {
            const unescapedFile = this.pathManager.joinedDestPath(relDir, defs_1.default.dest.dirTitleFile);
            yield mfs.writeFileAsync(unescapedFile, title);
            const escapedFile = this.pathManager.joinedDestPath(relDir, defs_1.default.dest.dirTitleHtmlFile);
            yield mfs.writeFileAsync(escapedFile, escapeHTML(title));
        });
    }
}
exports.default = TitleManager;
//# sourceMappingURL=titleManager.js.map